% XiedeSouza95_RobFiltEx
%
% This file duplicates the results in the Xie and de Souza paper
% using our robust filter design code.
% 
% Reference: 
% On robust filtering for linear systems with parameter uncertainty.
% by L. Xie and C.E. de Souza, 1995 CDC, p.2087-2092.
%
% PJS 6/25/2010  Initial Coding

%------------------------------------------------------------------------
% Problem Data (Section 3 of reference)
% Create P such that
%   [v;y;z] = P*[w;d]
% Punc wraps w = Delta*v around P, i.e.
%   [y;z] = Punc(Delta)*d   where Punc = lft(Delta,P)
%
% Note--In this example it can easily be shown that the filter
% F=1 achieves a gain of 1. It turns out that this is optimal.
% Our robust filter design comes very close to this optimal design
% but the results reported in Xie/de Souza only get a gain = 1.255.
%------------------------------------------------------------------------
del = 0.3*ureal('del',0);
A = [0 -1+del; 1 -0.5];

B1 = [-0.4545; 0.9090];
B2 = [0;0];
B = [B1 B2];

C1 = [0 100];
C2 = [0 100];
C = [C1;C2];

D1 = [0 1];
D2 = [0 0];
D = [D1;D2];

Punc = ss(A,B,C,D);

%------------------------------------------------------------------------
% Gain from Xie/de Souza:  Robust Hinfty filter computed using their 
% Ricatti equation approach
%------------------------------------------------------------------------
fprintf('Xie/de Souza: L2-L2 gain = %4.3f \n\n',1.255);

%------------------------------------------------------------------------
% Use freq. gridding to compute lower bound on optimal L2-L2 gain 
%------------------------------------------------------------------------
nomega = 250;
omega = logspace(-1,1,nomega);
nmeas = 1;
[gain1,Fopt1,PIopt1] = robfiltlb(Punc,nmeas,omega);
[gain1lb,mxidx1]=max(gain1);
fprintf('Freq. Grid: Lower Bound on minimal L2-L2 gain = %4.3f\n',gain1lb);

% Verify that IQC condition is satisfied at all frequencies
Nz = 1; Nw = 1; Nd = 2; Ny = nmeas;
Puncfr = frd(Punc,omega);
[Pfr,Delta] = lftdata(Puncfr.ResponseData);
[Nw,Nv]=size(Delta);
lmax = zeros(nomega,1);
for i1=1:nomega
    %GF = blkdiag(eye(Nv),[-Fopt1(:,:,i1) eye(Nz)])*Pfr(:,:,i1);
    %IQCLMI = [GF; eye(Nw+Nd)]'*PIopt1(:,:,i1)*[GF;eye(Nw+Nd)];
    %lmax(i1) = max(real(eig(IQCLMI)));
    
    % Get blocks of P
    P1 = Pfr(1:Nv,:,i1);
    P1I = [P1; eye(Nw) zeros(Nw,Nd)];
    P2 = Pfr(Nv+1:Nv+Ny,:,i1);
    P3 = Pfr(Nv+Ny+1:end,:,i1);    
    
    FPI = [-Fopt1(:,:,i1)*P2+P3; zeros(Nd,Nw) eye(Nd)];
    M = blkdiag( eye(Nz), -gain1(i1)^2*eye(Nd) );
            
    IQCLMI = P1I'*PIopt1(:,:,i1)*P1I + FPI'*M*FPI;
    lmax(i1) = max(real(eig(IQCLMI)));           
end
fprintf(['Max eval from IQC condition = ' num2str(max(lmax))]);
fprintf('  (should be < 0) \n\n');

%------------------------------------------------------------------------
% Compute a lower bound by gridding on the uncertainty space
% (This method is used in papers/thesis by K. Sun and Packard)
%------------------------------------------------------------------------
nunc = 20;
ncon = 1;
delgrid = linspace(-1,1,nunc);
gain2 = zeros(nunc,1);
for i1=1:nunc
    Pd = usubs(Punc,'del',delgrid(i1));
    M = [Pd(2,:) -1; Pd(1,:) 0];
    [K,CL,GAM,INFO] = hinfsyn(M,nmeas,ncon);
    gain2(i1) = GAM;
end
gain2lb = max(gain2);
fprintf('Unc. Grid: Lower Bound on minimal L2-L2 gain = %4.3f\n\n',gain2lb);

%------------------------------------------------------------------------
% Compute upper bound using DKSYN
%------------------------------------------------------------------------

% Scale plant by d-->e gain to get mu approximately equal to  1
% Note: DKYSN default treats delta as a complex parameter uncertainty.
% This is conservative, but turning on the Mixed Mu code actually does
% worse (larger value for optimal mu)
Munc = [Punc(2,:) -1; Punc(1,:) 0];
scl = diag([0.2054 1]); 
opt = dkitopt;
%opt = dkitopt('MixedMU','on');  % Mixed mu errors out on this problem
[Fdk,CL,GAM,DKINFO] =dksyn(scl*Munc,nmeas,ncon,opt);
gain3ub = 1/scl(1,1);

% Use WCGAIN to compute gain achieved by DKSYN filter
% WCGAIN treats delta as a real parameter uncertainty so gain4ub should
% be <= gain3ub.
GF = [-Fdk 1]*Punc;
GFfr = frd(GF,omega);
[MAXGAIN4,MAXGAINUNC4,WCINFO4] = wcgain(GFfr);
gain4ub = MAXGAIN4.UpperBound;
fprintf('DKSYN: L2-L2 gain = %4.3f \n\n',gain4ub);

%------------------------------------------------------------------------
% Compute upper bound using ROBFILTSYN
%------------------------------------------------------------------------
fprintf('Beginning Frequency-Gridding Robust Filter Synthesis \n');
% Fbasis = {tf(1,[1 0.1]) tf(1,[1 1]), tf(1,[1 10])};
% Fbasis = [Fbasis {tf(1) tf(1,[1 2*0.8*0.1 0.64])}];
Fbasis = {tf(1) tf(1,[1 0.1])};
tic
[gopt,Fopt5,PIopt,xopt] = robfiltsyn(Punc,Fbasis,nmeas,omega);
trfd= toc;
gain5ub = gopt;
fprintf('Comp time = %4.3f \n',trfd);
fprintf('Freq. Grid Syn: L2-L2 gain = %4.3f \n',gain5ub);

% Use WCGAIN to compute gain achieved by DKSYN filter
% Should get gain5ub = gain6ub;
GF = [-Fopt5 1]*Punc;
GFfr = frd(GF,omega);
[MAXGAIN6,MAXGAINUNC6,INFO6] = wcgain(GFfr);
gain6ub = MAXGAIN6.UpperBound;
fprintf('WCGAIN: L2-L2 gain for freq. grid filter = %4.3f \n',gain6ub);
fprintf(' (Two gains above should be equal)\n\n');

