% [copt,xopt,Zopt,info] = min_sdpha(F0,Fi,blk,c,opt,x0,Z0)
%
% Solves the primal semidefinite program:
%      Minimize  c'*x      subject to  x1*Fi{1} + .... + xn*Fi{n} < F0  
% and its dual:
%      Maximize  -Tr[F0*Z] subject to  Z=Z' >0 
%                                      -Tr[Fi{k}*Z]=c(k), k=1,...,n
% x is the vector of decision variables and Z is the symmetric matrix
% of dual variables. The function solves the LMI using SDPHA.
%
% Input:
%  F0        m by m symmetric matrix
%  FI        n by 1 cell array of m by m symmetric matrices
%  BLK       A vector containing the dimensions of the 
%               blocks in F0 and Fi.  If F0 and Fi have no block
%               structure, blk = m.  
%  C         m by 1 cost vector 
%  OPT       optional: The key options which can be set are:
%               (1) opt.prt:  When prt=0, the trace of execution is
%                   turned off 
%               (2) opt.abstol: NOT USED
%               (3) opt.reltol: Desired relative accuracy 
%               (4) opt.maxiter:  Maximum number of iterations 
%               Note 1: This is only a subset of the options for
%               SDPHA. The SDPHA manual lists many other
%               options which can be set as follows:  If
%               the SDPHA manual refers to the option 'foo',
%               set this option with 'opt.foo'.
%               Note 2: Advanced options take precedence. For
%               example, setting opt.prt and opt.printyes has 
%               the same effect (see SDPHA manual).  In this case
%               the advanced option, opt.printyes takes precedence.
%  X0        optional: Initial primal variable
%  Z0        optional: Initial dual variable
%
% Output:
%  COPT      2x1 vector, COPT(1) is the final primal cost, c'*x,
%            and COPT(2) is the final dual cost, -Tr[F0*Z]
%  XOPT      final value of the vector x of primal variables.
%  ZOPT      final value of dual variables
%  INFO      Additional information from SDPHA:
%            (1) info.gaphist: history of primal-duality gap 
%            (2) info.infeashist: history of infeasibility measure
%            (3) info.preshist: history of closeness to central path
%            (4) info.info: vector containing the following info:
%                   [iterations, final gap, cpu time, flops]
%            (5) info.solvertime: Not an SDPHA output.  This is the
%                cputime actually spent in the SDPHA optimization.
%                This is different from value in info.info because 
%                it includes all time SDPHA takes to set up the 
%                optimization once its function is called.
%            See SDPHA manual for more details. 
%
% See also: 
%
% Coding:   6/30/2001   Pete Seiler

function [copt,xopt,Zopt,info] = min_sdpha(F0,Fi,blk,c,opt,x0,Z0)

% Is function called correctly?
if nargin <4 | nargout > 4
  error(['usage: [copt,xopt,Zopt,info] = ' ...
    'min_sdpha(F0,Fi,blk,c,{opt},{x0},{Z0})']);
end
if nargin < 7
  Z0=[];
  if nargin <6
    x0=[];
    if nargin <5
      opt = [];
    end
  end
end

% Is SDPHA installed and in Matlab's search path?
if exist('sdpha','file')~=2
  error( sprintf(['SDPHA is not in Matlab''s search path. \n' ... 
  'To download SDPHA, go to: \n' ...
  '\t http://members-http-3.rwc1.sfba.home.net/nathan-brixius/SDPHA/']));
end

% Set SDPHA options  (startup.m is an SDPHA function, but
%  some users may have already have a startup.m file.
%  If the SDPHA version of startup.m does not take precedence,
%  then set the necessary global variables here)
blkin = blk; clear blk;
dir1=which('sdpha');
dir2=which('startup');
if strcmp(dir1,dir2)
  startup
else
  global cmp blk vers gam predcorr expon rho use_corrprim ...
          gaptol steptol maxit printyes infeaspt_opt infeatol;
  gam          = 0.0;    % 0 means using default parameter.  
  vers         = 2;      % HKM dir.
  predcorr     = 1;
  expon        = [2 1 1 2 1 1 1 1 1 1 1 1 1 1 ]; 
  rho          = inf; 
  use_corrprim = 1;
  infeatol     = 1e-15;
  gaptol       = 1e-9;
  steptol      = 1e-6; 
  maxit        = 50; 
  printyes     = 0; 
  infeaspt_opt = 1;  
end
cmp = 0;     % No complex data
blk = blkin; 
fldnames = fieldnames(opt);
for i1 =1:length(fldnames)
  eval([fldnames{i1} '= opt.' fldnames{i1} ';']);
end
if ~isfield(opt,'printyes'); printyes = opt.prt;         end
if ~isfield(opt,'gaptol');   gaptol = opt.reltol;        end
if ~isfield(opt,'maxit');    maxit = opt.maxiter;        end
if printyes == 0; warning off; end

% Solve with SDPHA
cputime0=cputime;
[obj,Zopt,xopt,slackvar,gaphist,infeashist,preshist,sdphainfo] = ...
   sdpha(Fi,F0,-c);
info.solvertime = cputime-cputime0;

% Set some output variables
warning on;
info.gaphist = gaphist;
info.infeashist = infeashist;
info.preshist = preshist;
info.info = sdphainfo;
copt = [c'*xopt -trace(F0*Zopt)];



