% [copt,xopt,Zopt,info] = min_sedumi(F0,Fi,blk,c,opt,x0,Z0)
%
% Solves the primal semidefinite program:
%      Minimize  c'*x      subject to  x1*Fi{1} + .... + xn*Fi{n} < F0  
% and its dual:
%      Maximize  -Tr[F0*Z] subject to  Z=Z' >0 
%                                      -Tr[Fi{k}*Z]=c(k), k=1,...,n
% x is the vector of decision variables and Z is the symmetric matrix
% of dual variables. The function converts from the canonical format 
% to SeDuMi format and then solves the LMI using SeDuMi.
%
% Input:
%  F0        m by m symmetric matrix
%  FI        n by 1 cell array of m by m symmetric matrices
%  BLK       A vector containing the dimensions of the 
%               blocks in F0 and Fi.  If F0 and Fi have no block
%               structure, blk = m.  
%  C         m by 1 cost vector 
%  OPT       optional: The key options which can be set are:
%               (1) opt.prt:  When prt=0, the trace of execution is
%                   turned off 
%               (2) opt.abstol: Desired absolute accuracy 
%               (3) opt.reltol: NOT USED
%               (4) opt.maxiter:  Maximum number of iterations 
%               Note 1: This is only a subset of the options for
%               Sedumi. The Sedumi manual lists many other
%               options which can be set as follows:  If
%               the Sedumi manual refers to the option 'pars.foo',
%               set this option with 'opt.foo'.
%               Note 2: Advanced options take precedence. For
%               example, setting opt.prt and opt.fid has the
%               same effect (see Sedumi manual).  In this case
%               the advanced option, opt.fid takes precedence.
%  X0        optional: Initial primal variable
%  Z0        optional: Initial dual variable
%
% Output:
%  COPT      2x1 vector, COPT(1) is the final primal cost, c'*x,
%            and COPT(2) is the final dual cost, -Tr[F0*Z]
%  XOPT      final value of the vector x of primal variables.
%  ZOPT      final value of dual variables
%  INFO      Additional information from Sedumi:
%            (1) info.cpusec: solution time 
%            (2) info.iter: number of iterations
%            (2) info.feasratio: final value of feasibility indicator.
%            (3) info.pinf, info.dinf: feasibility status of primal
%                    dual pair.  If pinf=dinf=0, then optimal solution 
%                    was found.  If pinf=1,then primal is infeasible.  
%                    If dinf=1, then dual is infeasible.
%            (4) info.numerr: measure of numerical problems.  If
%                    numerr=0, then desired accuracy, pars.eps, was
%                    achieved.  If numerr=1, then Sedumi only
%                    achieved the accuracy specified by pars.bigeps
%                    (default: 1e-3).
%                    If numerr=2, then Sedumi failed completely due
%                    to numerical problems.
%            (5) info.solvertime: Not a Sedumi output.  This is the
%                cputime actually spent in the Sedumi optimization.
%                This is different from info.cpusec because it 
%                includes all time Sedumi takes to set up the 
%                optimization once its function is called.
%            See Sedumi manual for more details. 
%
% See also: 
%
% Coding:   6/29/2001   Pete Seiler

% 10/8/2001:  Patch suggested by D. Cox:
%   Preallocate space and store variables sparsely (instead of fully).


function [copt,xopt,Zopt,info] = min_sedumi(F0,Fi,blk,c,opt,x0,Z0)

% Is function called correctly?
if nargin <4 | nargout > 4
  error(['usage: [copt,xopt,Zopt,info] = ' ...
    'min_sedumi(F0,Fi,blk,c,{opt},{x0},{Z0})']);
end
if nargin < 7
  Z0=[];
  if nargin <6
    x0=[];
    if nargin <5
      opt = [];
    end
  end
end

% Is SeDuMi installed and in Matlab's search path?
if exist('sedumi','file')~=2
  error( sprintf(['SeDuMi is not in Matlab''s search path. \n' ... 
	 'To download SeDuMi, go to: \n' ...
         '\t http://fewcal.kub.nl/sturm/software/sedumi.html']) );
end

% Convert from canonical form to Sedumi form
F0vec = [];Fivec = []; 
base = 0; 
for i1 = 1:length(blk);
  idx = base+[1:blk(i1)];
  % vectorize and store blocks:
  F0vec = [F0vec; sparse(vec( F0(idx,idx) )) ];
  
  % fi=[];
  fi=zeros(length(idx)^2,size(Fi,1));
  for i2 = 1:size(Fi,1);
     %fi(:,i2) = sparse(vec( Fi{i2}(idx,idx) ));
     fi(:,i2) = vec( Fi{i2}(idx,idx) );
  end     
  Fivec = [Fivec; sparse(fi)];
  
  base = base+blk(i1);
end
K.s = blk;

% Set SeDuMi options
pars = opt;
if ~isfield(opt,'fid'); pars.fid = opt.prt;     end
if ~isfield(opt,'eps'); pars.eps = opt.abstol;  end

% Solve with SeDuMi
cputime0=cputime;
[xdual,xopt,info]=sedumi(Fivec,-c,F0vec,K,pars);
info.solvertime = cputime-cputime0;

% Convert dual variables back to matrix form
Zopt = [];base =0;
for i1 = 1:length(blk)
  idx = base+[1:blk(i1)^2];
  Zopt = blkdiag(Zopt, mat(xdual(idx)));
  base = base+blk(i1)^2;
end
copt = [c'*xopt -trace(F0*Zopt)];



